<?php

namespace Drupal\block_entity_form;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

class EntityFormService {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  private EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  private EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Constructs the service
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entityDisplayRepository
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityTypeBundleInfoInterface $entityTypeBundleInfo, EntityDisplayRepositoryInterface $entityDisplayRepository) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityTypeBundleInfo = $entityTypeBundleInfo;
    $this->entityDisplayRepository = $entityDisplayRepository;
  }

  /**
   * Gets the available content types and bundles
   *
   * @return array
   */
  public function getContentTypesOptions() {
    /** @var \Drupal\Core\Entity\ContentEntityTypeInterface[] $content_entity_types */
    $content_entity_types = [];
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type_definition) {
      if ($entity_type_definition->getGroup() == 'content') {
        $content_entity_types[$entity_type_id] = $entity_type_definition;
      }
    }
    $options = [];
    foreach ($content_entity_types as $type_key => $entity_type) {
      if (!$entity_type->hasFormClasses() || $type_key == 'comment') {
        continue;
      }
      $entity_type_bundles = $this->entityTypeBundleInfo->getBundleInfo($type_key);
      foreach ($entity_type_bundles as $bundle_key => $bundle_info) {
        if ($bundle_key == 'personal' && $type_key == 'contact_message') {
          continue;
        }
        $options[(string) $entity_type->getLabel()][$type_key . '.' . $bundle_key] = $bundle_info['label'];
      }
    }
    return $options;
  }

  /**
   * Gets form modes for a given entity type
   *
   * @param $entityTypeId
   *
   * @return array
   */
  public function getFormModeOptions($entityTypeId) {
    return $this->entityDisplayRepository->getFormModeOptions($entityTypeId);
  }

}
